import { system, world } from "@minecraft/server";
import { ActionFormData,ModalFormData } from "@minecraft/server-ui";
// Constants
const CONFIG = {
    kits_enabled: world.getDynamicProperty('Kits_Enabled'),
    max_kits: 100,
    KITS_PROPERTY_NAME: "kits", // Property name for storing kits
    MoneySystemName: world.getDynamicProperty('MoneySystemName') // Get the scoreboard objective name from dynamic property
};

// Initialize kit storage from world dynamic property
let kitsJson = world.getDynamicProperty(CONFIG.KITS_PROPERTY_NAME);
let kits = {};

// Check if kitsJson is defined and is a valid JSON string
if (kitsJson && kitsJson !== 'undefined') {
    try {
        kits = JSON.parse(kitsJson);
    } catch (error) {
        // Handle the case where the JSON is malformed
        console.error("Failed to parse kits data, initializing to empty:", error);
        kits = {};
    }
} else {
    // If undefined, initialize kits to an empty object
    kits = {};
}


// Save a kit
function saveKit(player, kitName, kitPrice) {
    // Check if the player is an admin
    if (!player.hasTag('Admin') && !player.hasTag('admin')) {
        player.sendMessage("Error: You do not have permission to save kits.");
        return;
    }

    const inventory = player.getComponent("inventory").container;

    if (kits[kitName]) {
        player.sendMessage(`Error: Kit '${kitName}' already exists.`);
        return;
    }

    if (Object.keys(kits).length >= CONFIG.max_kits) {
        player.sendMessage("Error: Maximum number of kits reached.");
        return;
    }

    // Save all items in the player's inventory
    let items = [];
    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item) {
            items.push(item);
        }
    }

    kits[kitName] = {
        items: items,
        price: parseInt(kitPrice, 10)
    };

    // Update the dynamic property
    world.setDynamicProperty(CONFIG.KITS_PROPERTY_NAME, JSON.stringify(kits));

    player.sendMessage(`Kit '${kitName}' saved with price ${kitPrice}.`);
}

// List all kits
function listKits(player) {
    if (Object.keys(kits).length === 0) {
        player.sendMessage("There are no saved kits.");
        return;
    }

    let message = "Saved Kits:\n";
    for (const [name, data] of Object.entries(kits)) {
        message += `- ${name}: ${data.price}\n`;
    }
    player.sendMessage(message);
}

// Delete a kit
function deleteKit(player, kitName) {
    // Check if the player is an admin
    if (!player.hasTag('Admin') && !player.hasTag('admin')) {
        player.sendMessage("Error: You do not have permission to delete kits.");
        return;
    }

    if (!kits[kitName]) {
        player.sendMessage(`Error: Kit '${kitName}' does not exist.`);
        return;
    }

    delete kits[kitName];
    
    // Update the dynamic property
    world.setDynamicProperty(CONFIG.KITS_PROPERTY_NAME, JSON.stringify(kits));

    player.sendMessage(`Kit '${kitName}' has been deleted.`);
}

// Load a kit
function loadKit(player, kitName) {
    if (!kits[kitName]) {
        player.sendMessage(`Error: Kit '${kitName}' does not exist.`);
        return;
    }

    // Get the player's money using the scoreboard
    const playerMoney = world.scoreboard.getObjective(`${CONFIG.MoneySystemName}`).getScore(player);
    const kitPrice = kits[kitName].price;

    if (playerMoney === undefined) {
        player.sendMessage("Error: Money property not set for this player.");
        return;
    }

    if (playerMoney < kitPrice) {
        player.sendMessage(`Error: You need ${kitPrice - playerMoney} more money to buy this kit.`);
        return;
    }

    // Deduct money
    system.run(() => {
        world.scoreboard.getObjective(`${CONFIG.MoneySystemName}`).setScore(player, playerMoney - kitPrice);
    });

    // Clear the player's inventory
    const inventory = player.getComponent("inventory").container;
    player.runCommandAsync('clear @s');

    system.run(() => {
        let availableSlots = 0;

        // Check how many items can fit in the inventory
        kits[kitName].items.forEach((item) => {
            // Ensure the item is a valid ItemStack before adding it
            if (item && availableSlots < inventory.size) {
                inventory.setItem(availableSlots, item); // Set item at the available slot index
                availableSlots++; // Move to the next slot for the next item
            } else if (availableSlots >= inventory.size) {
                player.sendMessage("Error: Not enough inventory space to load all items from the kit.");
                return; // Stop processing if no more space is available
            }
        });
    });

    player.sendMessage(`Kit '${kitName}' has been loaded. You have been charged ${kitPrice}.`);
}



// Command handler
function onChatCommand(player, message) {
    const args = message.split(" ");
    const command = args[0].toLowerCase();

    switch (command) {
        case "!savekit":
            if (args.length < 3) {
                player.sendMessage("Usage: !savekit [kitname] [kitprice]");
                return;
            }
            saveKit(player, args[1], args[2]);
            break;
        case "!listkits":
            listKits(player);
            break;
        case "!deletekit":
            if (args.length < 2) {
                player.sendMessage("Usage: !deletekit [kitname]");
                return;
            }
            deleteKit(player, args[1]);
            break;
        case "!kit":
            if (args.length < 2) {
                player.sendMessage("Usage: !kit [kitname]");
                return;
            }
            loadKit(player, args[1]);
            break;
        default:

    }
}

// Event Listener for Chat Commands
world.beforeEvents.chatSend.subscribe(event => {
    if (event.message.startsWith("!")) {
        event.cancel = true; // Prevent the message from appearing in chat
        onChatCommand(event.sender, event.message);
    }
});



// Function to display the menu
function showKitsMenu(player) {
    // Create the action form for the kits
    const form = new ActionFormData()
        .title("Kit Management")
        .body("Select a kit to manage:")
        .button("Cancel"); // Cancel button

    // Add a button for each kit
    for (const kitName in kits) {
        form.button(kitName);
    }

    form.show(player).then(response => {
        if (response.canceled) return;

        // If a valid kit is selected
        const selectedKitName = Object.keys(kits)[response.selection - 1]; // Adjust for the 'Cancel' button
        if (selectedKitName) {
            showKitOptionsMenu(player, selectedKitName);
        }
    });
}

// Function to show options for a selected kit
function showKitOptionsMenu(player, kitName) {
    const optionsForm = new ActionFormData()
        .title(`Manage Kit: ${kitName}`)
        .button("Edit")
        .button("Delete")
        .button("Cancel");

    optionsForm.show(player).then(response => {
        if (response.canceled) return;

        switch (response.selection) {
            case 0:
                showEditKitMenu(player, kitName);
                break;
            case 1:
                deleteKit(player, kitName);
                break;
        }
    });
}

// Function to show the edit menu for a kit
function showEditKitMenu(player, kitName) {
    const kit = kits[kitName];
    const editForm = new ModalFormData()
        .title(`Edit Kit: ${kitName}`)
        .textField("Kit Name", "Enter new name", kitName)
        .textField("Price", "Enter new price", String(kit.price));

    editForm.show(player).then(data => {
        if (data.canceled) return;

        const newKitName = data.formValues[0];
        const newPrice = parseInt(data.formValues[1], 10);

        if (!isNaN(newPrice) && newPrice > 0) {
            delete kits[kitName]; // Remove the old kit
            saveKit(player, newKitName, newPrice); // Save the kit with the new name and price
            player.sendMessage(`Kit '${kitName}' has been updated to '${newKitName}' with price ${newPrice}.`);
        } else {
            player.sendMessage("§cError: Please enter a valid number for the price.");
        }
    });
}

system.afterEvents.scriptEventReceive.subscribe((data) => {
    
    if (data.id === 'realm:kits') {
        showKitsMenu(data.sourceEntity)
    }
})